@@ -1,7 +1,7 @@
 /*
  *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 161 $, $Date: 2007-05-05 22:53:24 +0100 (Sat, 05 May 2007) $
- *  Copyright 2007 RW Penney
+ *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  (C)Copyright 2007-2008, RW Penney
  */
 
 /*
@@ -12,18 +12,14 @@
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
-    As a special exemption, permission is granted to link cryptmount
-    with the OpenSSL project's "OpenSSL" library and distribute
-    the linked code without invoking clause 2(b) of the GNU GPL version 2.
-
     cryptmount is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with cryptmount; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <config.h>
@@ -149,6 +145,7 @@ static int kmblti_needs_pw(const keyinfo_t *keyinf)
 
 
 static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
+            const km_pw_context_t *pw_ctxt,
             unsigned char **key, int *keylen, FILE *fp_key)
     /* extract key from sha1/blowfish encrypted file */
 {   cm_bf_ctxt_t *ctxt;
@@ -162,13 +159,8 @@ static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
 
     *key = NULL; *keylen = 0;
 
-#ifdef TESTING
-    passwd = (char*)malloc((size_t)1024);
-    strncpy(passwd, (test_ctxtptr->argpassword[0] != NULL ? test_ctxtptr->argpassword[0] : ""),
-            1024);
-#else
-    if (km_get_passwd(ident, &passwd, 0, 0) != ERR_NOERROR) goto bail_out;
-#endif
+    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 0, 0);
+    if (eflag != ERR_NOERROR) goto bail_out;
 
     /* read key header: */
     hbuff[0] = '\0';
@@ -226,9 +218,17 @@ static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
 
     /* verify checksum: */
     if (!km_aug_verify(buff, (unsigned)*keylen, &chksum0, &chksum)) {
-        fprintf(stderr, "checksum mismatch in keyfile (builtin, %x != %x)\n",
-            (unsigned)chksum, (unsigned)chksum0);
-        eflag = ERR_BADFILE;
+        switch (pw_ctxt->debug_level) {
+            case 0:
+                fprintf(stderr, _("password mismatch when extracting key\n"));
+                break;
+            case 1:     /* fall through... */
+            default:
+                fprintf(stderr, "checksum mismatch in keyfile (builtin, %x != %x)\n",
+                        (unsigned)chksum, (unsigned)chksum0);
+                break;
+        }
+        eflag = ERR_BADDECRYPT;
     }
 
     if (keyinfo->maxlen > 0 && *keylen > keyinfo->maxlen) {
@@ -240,18 +240,20 @@ static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
     if (ferror(fp_key) != 0) {
         fprintf(stderr, _("key-extraction failed for \"%s\"\n"),
                 keyinfo->filename);
-        eflag = ERR_BADDECRYPT;
+        eflag = ERR_BADFILE;
     }
 
   bail_out:
 
-    if (buff != NULL) sec_free(buff);
+    if (buff != NULL) sec_free((void*)buff);
+    if (passwd != NULL) sec_free((void*)passwd);
 
     return eflag;
 }
 
 
 static int kmblti_put_key(const char *ident, const keyinfo_t *keyinfo,
+            const km_pw_context_t *pw_ctxt,
             const unsigned char *key, const int keylen, FILE *fp_key)
     /* store key in sha1/blowfish encrypted file */
 {   cm_bf_ctxt_t *ctxt;
@@ -261,13 +263,8 @@ static int kmblti_put_key(const char *ident, const keyinfo_t *keyinfo,
     size_t buffsz, wrcnt;
     int cnt, eflag=ERR_NOERROR;
 
-#ifdef TESTING
-    passwd = sec_realloc((void*)passwd, (size_t)1024);
-    strncpy(passwd, (test_ctxtptr->argpassword[1] != NULL ? test_ctxtptr->argpassword[1] : ""),
-            1024);
-#else
-    if (km_get_passwd(ident, &passwd, 1, 1) != ERR_NOERROR) goto bail_out;
-#endif
+    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 1, 1);
+    if (eflag != ERR_NOERROR) goto bail_out;
 
     /* write key header: */
     wrcnt = 0;
@@ -321,8 +318,8 @@ static int kmblti_put_key(const char *ident, const keyinfo_t *keyinfo,
     }
 
   bail_out:
-    if (buff != NULL) sec_free(buff);
-    if (passwd != NULL) sec_free(passwd);
+    if (buff != NULL) sec_free((void*)buff);
+    if (passwd != NULL) sec_free((void*)passwd);
 
     return eflag;
 }
@@ -379,5 +376,5 @@ keymanager_t *kmblti_gethandle(void)
 }
 
 /*
- *  (C)Copyright 2007, RW Penney
+ *  (C)Copyright 2007-2008, RW Penney
  */
