@@ -1,7 +1,7 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  $Revision: 170 $, $Date: 2007-07-15 17:53:26 +0100 (Sun, 15 Jul 2007) $
- *  Copyright 2005-2007 RW Penney
+ *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  (C)Copyright 2005-2008, RW Penney
  */
 
 /*
@@ -12,18 +12,14 @@
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
-    As a special exemption, permission is granted to link cryptmount
-    with the OpenSSL project's "OpenSSL" library and distribute
-    the linked code without invoking clause 2(b) of the GNU GPL version 2.
-
     cryptmount is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with cryptmount; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <config.h>
@@ -65,7 +61,7 @@
 #endif
 
 
-typedef struct targelt
+typedef struct targelt          /* element of filesystem-target list */
 {
     const cment_t *entry;
 
@@ -73,7 +69,7 @@ typedef struct targelt
 } targelt_t;
 
 
-typedef enum
+typedef enum                    /* top-level operating mode */
 {
     M_DEFAULT, M_HELP,
     M_PREPARE, M_RELEASE,
@@ -87,17 +83,19 @@ typedef enum
 
 static int64_t getblkcount(const char *device);
 static int execute_list(cmmode_t mode, const cment_t *cmtable,
+                const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist);
 static int do_list(const cment_t *cment);
-static int do_devsetup(const cment_t *cment, char **mntdev);
+static int do_devsetup(const km_pw_context_t *pw_ctxt,
+                const cment_t *cment, char **mntdev);
 static int do_devshutdown(const cment_t *cment);
-static int do_mount(const cment_t *cment);
+static int do_mount(const km_pw_context_t *pw_ctxt, const cment_t *cment);
 static int do_unmount(const cment_t *cment);
-static int do_swapon(const cment_t *cment);
+static int do_swapon(const km_pw_context_t *pw_ctxt, const cment_t *cment);
 static int do_swapoff(const cment_t *cment);
-static int do_passwd(const cment_t *cment);
-static int do_keygen(const cment_t *cment, const char *params,
-                int reuse, const cment_t *cmtable);
+static int do_passwd(const km_pw_context_t *pw_ctxt, const cment_t *cment);
+static int do_keygen(const km_pw_context_t *pw_ctxt, const cment_t *cment,
+                const char *params, int reuse, const cment_t *cmtable);
 static int do_safetynet();
 
 
@@ -118,6 +116,7 @@ usage: cryptmount [OPTION [target ...]]\n\
     --prepare <target>\n\
     --release <target>\n\
     --config-fd <num>\n\
+    --passwd-fd <num>\n\
     --swapon <target>\n\
     --swapoff <target>\n\
     --version\n\
@@ -220,6 +219,11 @@ static int64_t getblkcount(const char *device)
 static int do_list(const cment_t *cment)
     /* print list of available filing-system targets */
 {
+    /* TRANSLATORS: this string is marked as 'no-c-format' because
+       some localizations may require the mount-point and filesystem type
+       to be printed in a different order, but the untranslated string needs
+       to remain an ordinary string that can be printed without gettext. */
+    /* xgettext:no-c-format */
     printf(_("%-16s  [to mount on \"%s\" as \"%s\"]\n"),
             cment->ident, cment->dir, cment->fstype);
 
@@ -227,7 +231,8 @@ static int do_list(const cment_t *cment)
 }
 
 
-static int do_devsetup(const cment_t *cment, char **mntdev)
+static int do_devsetup(const km_pw_context_t *pw_ctxt,
+                    const cment_t *cment, char **mntdev)
     /* setup all devices needed to access encrypted target */
 {   enum { BUFFMIN=1024 };
     unsigned char *key=NULL;
@@ -238,7 +243,7 @@ static int do_devsetup(const cment_t *cment, char **mntdev)
     char *dmparams=NULL, *tgtdev=NULL, *loopdev=NULL;
 
     /* get crypto-key for filing system: */
-    eflag = get_key(cment->ident, &cment->key, &key, &keylen);
+    eflag = get_key(cment->ident, &cment->key, pw_ctxt, &key, &keylen);
     if (eflag != ERR_NOERROR) {
         fprintf(stderr, _("failed to extract cipher key\n"));
         goto bail_out;
@@ -268,6 +273,7 @@ static int do_devsetup(const cment_t *cment, char **mntdev)
         }
         eflag = loop_setup(loopdev, cment->dev,
                         (is_readonlyfs(cment->dev) ? O_RDONLY : O_RDWR));
+        if (eflag != ERR_NOERROR) goto bail_out;
         tgtdev = loopdev;
     } else {
         fprintf(stderr,
@@ -384,7 +390,7 @@ int do_devshutdown(const cment_t *cment)
 }
 
 
-static int do_mount(const cment_t *cment)
+static int do_mount(const km_pw_context_t *pw_ctxt, const cment_t *cment)
 {   int freedev=0,eflag=ERR_NOERROR;
     char *mntdev=NULL;
     tgtstat_t *tstat;
@@ -395,7 +401,7 @@ static int do_mount(const cment_t *cment)
         goto bail_out;
     }
 
-    eflag = do_devsetup(cment, &mntdev);
+    eflag = do_devsetup(pw_ctxt, cment, &mntdev);
     if (eflag != ERR_NOERROR) goto bail_out;
 
 #if WITH_FSCK
@@ -450,6 +456,9 @@ static int do_unmount(const cment_t *cment)
             fprintf(stderr, _("only \"%s\" can unmount \"%s\"\n"),
                 pwent->pw_name, cment->ident);
         } else {
+            /*  TRANSLATORS: the following expands to include
+                the *numerical* user-identity in place of '%lu',
+                e.g. giving 'only user-16 can unmount "target"': */
             fprintf(stderr, _("only user-%lu can unmount \"%s\"\n"),
                 tstat->uid, cment->ident);
         }
@@ -478,14 +487,14 @@ static int do_unmount(const cment_t *cment)
 }
 
 
-static int do_swapon(const cment_t *cment)
+static int do_swapon(const km_pw_context_t *pw_ctxt, const cment_t *cment)
 {   int freedev=0,eflag=ERR_NOERROR;
     char *mntdev=NULL;
     tgtstat_t *tstat;
 
 #if WITH_CSWAP
 
-    eflag = do_devsetup(cment, &mntdev);
+    eflag = do_devsetup(pw_ctxt, cment, &mntdev);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     if (fs_swapon(mntdev, cment) != ERR_NOERROR) {
@@ -561,7 +570,7 @@ static int do_swapoff(const cment_t *cment)
 }
 
 
-static int do_passwd(const cment_t *cment)
+static int do_passwd(const km_pw_context_t *pw_ctxt, const cment_t *cment)
 {   unsigned char *key=NULL;
     int keylen=0,eflag=ERR_NOERROR;
     char *newfname=NULL,*oldfname=NULL;
@@ -576,7 +585,7 @@ static int do_passwd(const cment_t *cment)
         goto bail_out;
     }
 
-    eflag = get_key(cment->ident, &cment->key, &key, &keylen);
+    eflag = get_key(cment->ident, &cment->key, pw_ctxt, &key, &keylen);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     sz = strlen(cment->key.filename) + 16;
@@ -591,7 +600,7 @@ static int do_passwd(const cment_t *cment)
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    eflag = put_key(cment->ident, &cment->key, key, keylen, fp);
+    eflag = put_key(cment->ident, &cment->key, pw_ctxt, key, keylen, fp);
     if (fclose(fp) != 0) eflag = ERR_BADFILE;
 
     if (eflag == ERR_NOERROR) {
@@ -628,8 +637,8 @@ static int do_passwd(const cment_t *cment)
 }
 
 
-static int do_keygen(const cment_t *cment, const char *params,
-                    int reuse, const cment_t *cmtable)
+static int do_keygen(const km_pw_context_t *pw_ctxt, const cment_t *cment,
+                    const char *params, int reuse, const cment_t *cmtable)
     /* create new filesystem crypto-key */
 {   unsigned char *key=NULL;
     int keylen=0, eflag=ERR_NOERROR;
@@ -665,7 +674,7 @@ static int do_keygen(const cment_t *cment, const char *params,
     }
 
     if (reuse) {
-        eflag = get_key(parent->ident, &parent->key, &key, &keylen);
+        eflag = get_key(parent->ident, &parent->key, pw_ctxt, &key, &keylen);
     } else {
         fprintf(stderr, _("generating random key; please be patient...\n"));
         key = (unsigned char*)sec_realloc(NULL, (size_t)keylen);
@@ -686,7 +695,7 @@ static int do_keygen(const cment_t *cment, const char *params,
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    eflag = put_key(cment->ident, &cment->key, key, keylen, fp);
+    eflag = put_key(cment->ident, &cment->key, pw_ctxt, key, keylen, fp);
     fclose(fp);
 
     if (eflag == ERR_NOERROR) {
@@ -725,7 +734,7 @@ static int do_safetynet()
      *  damage to filesystems that should have been properly removed
      *  by other mechanisms (e.g. --unmount, --swapoff) */
 
-    cment = alloc_cment();
+    cment = alloc_cment(NULL);
     old_ident = cment->ident;
 
     /* get list of all targets in status-file: */
@@ -875,11 +884,12 @@ static int check_mode(cmmode_t mode, const char *params,
 
 
 static int execute_list(cmmode_t mode, const cment_t *cmtable,
+                const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist)
-{   const targelt_t *elt;
+{   const targelt_t *elt=NULL;
     int i, prio=0, eflag=ERR_NOERROR;
     struct passwd *pwent=NULL;
-    const char *username, **keymgrs, *syslogmsg=NULL;
+    const char *username=NULL, **keymgrs=NULL, *syslogmsg=NULL;
 
     pwent = getpwuid(getuid());
     username = (pwent != NULL ? pwent->pw_name : "UNKNOWN");
@@ -918,7 +928,7 @@ static int execute_list(cmmode_t mode, const cment_t *cmtable,
                 break;
             case M_PREPARE:
                 syslogmsg = "prepare of \"%s\" by %s %s";
-                eflag = do_devsetup(elt->entry, NULL);
+                eflag = do_devsetup(pw_ctxt, elt->entry, NULL);
                 break;
             case M_RELEASE:
                 syslogmsg = "release of \"%s\" by %s %s";
@@ -929,7 +939,7 @@ static int execute_list(cmmode_t mode, const cment_t *cmtable,
             case M_MOUNT:
                 if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
                 syslogmsg = "mount of \"%s\" by %s %s";
-                eflag = do_mount(elt->entry);
+                eflag = do_mount(pw_ctxt, elt->entry);
                 break;
             case M_UNMOUNT:
                 if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
@@ -939,7 +949,7 @@ static int execute_list(cmmode_t mode, const cment_t *cmtable,
                 break;
             case M_SWAPON:
                 syslogmsg = "swapon \"%s\" by %s %s";
-                eflag = do_swapon(elt->entry);
+                eflag = do_swapon(pw_ctxt, elt->entry);
                 break;
             case M_SWAPOFF:
                 syslogmsg = "swapoff \"%s\" by %s %s";
@@ -949,17 +959,17 @@ static int execute_list(cmmode_t mode, const cment_t *cmtable,
             case M_PASSWORD:
                 if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
                 syslogmsg = "changing password for \"%s\" by %s %s";
-                eflag = do_passwd(elt->entry);
+                eflag = do_passwd(pw_ctxt, elt->entry);
                 break;
             case M_KEYGEN:
                 if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
                 syslogmsg = "key generation for \"%s\" by %s %s";
-                eflag = do_keygen(elt->entry, params, 0, NULL);
+                eflag = do_keygen(pw_ctxt, elt->entry, params, 0, NULL);
                 break;
             case M_KEYREU:
                 if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
                 syslogmsg = "key generation for \"%s\" by %s %s";
-                eflag = do_keygen(elt->entry, params, 1, cmtable);
+                eflag = do_keygen(pw_ctxt, elt->entry, params, 1, cmtable);
                 break;
             default:
                 break;
@@ -1034,6 +1044,7 @@ int main(int argc, char *argv[])
         { "key-managers",       no_argument,        NULL, (int)'k' },
         { "list",               no_argument,        NULL, (int)'l' },
         { "mount",              no_argument,        NULL, (int)'m' },
+        { "passwd-fd",          required_argument,  NULL, (int)'w' },
         { "prepare",            no_argument,        NULL, (int)'p' },
         { "release",            no_argument,        NULL, (int)'r' },
         { "reuse-key",          required_argument,  NULL, (int)'e' },
@@ -1052,10 +1063,11 @@ int main(int argc, char *argv[])
 #endif
     const char *params=NULL, *optlist="acdf:g:klmprsuvx";
     char *cmtab=NULL;
-    int val,idx,config_fd=-1,eflag=ERR_NOERROR;
+    int val, idx, config_fd=-1, passwd_fd=-1, eflag=ERR_NOERROR;
     cment_t *cmtable=NULL;
+    km_pw_context_t pw_ctxt;
     const cment_t *ent=NULL;
-    targelt_t *eltlist=NULL,**eltptr=&eltlist;
+    targelt_t *eltlist=NULL, **eltptr=&eltlist;
 
 #ifdef HAVE_GETTEXT
     /* setup internationalization of message-strings via gettext(): */
@@ -1066,7 +1078,8 @@ int main(int argc, char *argv[])
 
 #ifdef TESTING
     fprintf(stderr, "WARNING!!! cryptmount has been compiled for TESTING only - DO NOT INSTALL\n");
-    test_context.argconfigdir = test_context.argpassword[0] = test_context.argpassword[1] = NULL;
+    pw_ctxt.argpasswd[0] = pw_ctxt.argpasswd[1] = NULL;
+    test_context.argconfigdir = NULL;
 #endif
 
     if (mlockall(MCL_CURRENT | MCL_FUTURE) != 0) {
@@ -1114,13 +1127,15 @@ int main(int argc, char *argv[])
                 mode = M_UNMOUNT; break;
             case 'v':
                 mode = M_VERSION; break;
+            case 'w':
+                sscanf(optarg, "%d", &passwd_fd); break;
 #ifdef TESTING
             case 'D':
                 test_context.argconfigdir = optarg; break;
             case 'W':
-                test_context.argpassword[0] = optarg; break;
+                pw_ctxt.argpasswd[0] = optarg; break;
             case 'N':
-                test_context.argpassword[1] = optarg; break;
+                pw_ctxt.argpasswd[1] = optarg; break;
             case 'T':
                 return cm_run_tests(); break;
 #endif
@@ -1142,6 +1157,18 @@ int main(int argc, char *argv[])
         exit(EXIT_OK);
     }
 
+    /* configure source of passwords: */
+    if (passwd_fd >= 0) {
+        pw_ctxt.fd_pw_source = fdopen(passwd_fd, "r");
+        if (pw_ctxt.fd_pw_source == NULL) {
+            fprintf(stderr, _("bad file-descriptor (%d)\n"), passwd_fd);
+            exit(EXIT_BADOPT);
+        }
+    } else {
+        pw_ctxt.fd_pw_source = NULL;
+    }
+    pw_ctxt.debug_level = 0;
+
     /* (check &) read-in configuration file: */
 #ifndef TESTING
     if (sycheck_cmtab(cmtab) != ERR_NOERROR
@@ -1200,7 +1227,7 @@ int main(int argc, char *argv[])
 
 
     /* execute user-selected task: */
-    eflag = execute_list(mode, cmtable, params, eltlist);
+    eflag = execute_list(mode, cmtable, &pw_ctxt, params, eltlist);
     free_keymanagers();
 
 
@@ -1218,5 +1245,5 @@ int main(int argc, char *argv[])
 
 /*
  *  cryptmount.c
- *  (C)Copyright 2005-2007, RW Penney
+ *  (C)Copyright 2005-2008, RW Penney
  */
