@@ -1,7 +1,7 @@
 /*
  *  filesystem-related utilities for cryptmount
- *  $Revision: 154 $, $Date: 2007-03-31 16:55:38 +0100 (Sat, 31 Mar 2007) $
- *  Copyright 2005-2007, RW Penney
+ *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  (C)Copyright 2005-2008, RW Penney
  */
 
 /*
@@ -12,18 +12,14 @@
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
-    As a special exemption, permission is granted to link cryptmount
-    with the OpenSSL project's "OpenSSL" library and distribute
-    the linked code without invoking clause 2(b) of the GNU GPL version 2.
-
     cryptmount is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with cryptmount; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <config.h>
@@ -481,5 +477,5 @@ static int run_sucommand(const char *path, const char **argv,
 
 /*
  *  fsutils.c
- *  (C)Copyright 2005-2007, RW Penney
+ *  (C)Copyright 2005-2008, RW Penney
  */
