@@ -1,7 +1,7 @@
 /*
  *  declarations for miscellaneous utilities for cryptmount
  *  $Revision: 134 $, $Date: 2007-02-19 17:18:07 +0000 (Mon, 19 Feb 2007) $
- *  Copyright 2005-2007, RW Penney
+ *  (C)Copyright 2005-2008, RW Penney
  */
 
 /*
@@ -12,18 +12,14 @@
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
-    As a special exemption, permission is granted to link cryptmount
-    with the OpenSSL project's "OpenSSL" library and distribute
-    the linked code without invoking clause 2(b) of the GNU GPL version 2.
-
     cryptmount is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with cryptmount; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 
@@ -31,6 +27,16 @@
 #define _UTILS_H
 
 #include <inttypes.h>
+#include <stdio.h>
+
+typedef struct km_pw_context {
+    FILE *fd_pw_source;         /* stream from which to read passwords */
+    unsigned debug_level;	/* verbosity of debugging information */
+#ifdef TESTING
+    const char *argpasswd[2];   /* password(s) passed via command-line */
+#endif
+} km_pw_context_t;
+
 
 int cm_path(char **buff, const char *file);
 char *cm_strdup(const char *orig);
@@ -41,10 +47,11 @@ void mem_cleanse(unsigned char *addr, size_t sz);
 void sec_free(void *ptr);
 
 int get_randkey(unsigned char *key, size_t len);
-int km_get_passwd(const char *ident, char **passwd, int isnew, int verify);
+int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
+                char **passwd, int isnew, int verify);
 unsigned km_aug_keysz(unsigned keylen, unsigned blksz);
 unsigned char *km_aug_key(const unsigned char *key, unsigned keylen,
-                    unsigned blocksz, size_t *buffsz);
+                unsigned blocksz, size_t *buffsz);
 int km_aug_verify(const unsigned char *buff, unsigned keylen,
                 uint32_t *expected, uint32_t *actual);
 
@@ -65,5 +72,5 @@ void cm_sha1_free(cm_sha1_ctxt_t *ctxt);
 
 /*
  *  utils.h
- *  (C)Copyright 2005-2007, RW Penney
+ *  (C)Copyright 2005-2008, RW Penney
  */
